import React, { PropsWithChildren, ReactElement, useEffect, useRef, useState } from 'react';

import {
    Autocomplete,
    AutocompleteRenderInputParams,
    ListSubheader,
    Popper,
    useMediaQuery,
    useTheme,
} from '@mui/material';

import { LineLoader } from './helper/LineLoader';
import { RecentClearAll } from './helper/RecentClearAll';
import { renderGroupBy, renderOptionLabel } from './helper/renderGroupName';
import { SeeMoreButton } from './helper/SeeMoreButton';
import { StyledPopper, StyledTextField } from './NsAutocomplete.styled';
import { ComponentBoxProps, NsAutocompleteProps, OptionsProps } from './types.d';
import { VariableSizeList, ListChildComponentProps } from 'react-window';

const LISTBOX_PADDING = 8;
const noop = () => {};
const OuterElementContext = React.createContext({});

const NsAutocomplete = ({
    id = 'custom_autocomplete_dropdown',
    options,
    OptionsUI,
    loading,
    noOptionsText,
    onInputChange,
    onChange,
    onFocus,
    clear,
    seeMore,
    placeholder,
    className,
    iconStart,
    iconEnd,
    inputValue,
    value,
    isFilterEnabled = true,
    fullWidth,
    onKeypress,
}: PropsWithChildren<NsAutocompleteProps>): ReactElement => {
    const listboxRef = useRef<any | null>(null);
    const [scrollTopPosition, setScrollTopPosition] = useState(0);

    useEffect(() => {
        if (listboxRef.current && options.length > 0) {
            listboxRef.current.scrollTop = scrollTopPosition;
        }
    }, [options, scrollTopPosition]);

    const OuterElementType = React.forwardRef((props: any, ref) => {
        const outerProps = React.useContext(OuterElementContext);
        return <ul ref={ref} {...props} {...outerProps} />;
    });

    OuterElementType.displayName = 'OuterElementType';

    const renderInput = (params: AutocompleteRenderInputParams) => (
        <StyledTextField
            {...(params as any)}
            placeholder={placeholder}
            InputProps={{
                ...params.InputProps,
                className: 'styled-text-field',
                startAdornment: iconStart,
                endAdornment: iconEnd,
            }}
        />
    );

    const renderOption = (props: ComponentBoxProps, option: OptionsProps) =>
        OptionsUI ? (
            <OptionsUI {...props} option={option} key={props.id} />
        ) : (
            <li {...(props as any)} key={props.id}>
                {option.value}
            </li>
        );

    const renderRow = (props: ListChildComponentProps) => {
        const { data, index, style } = props;
        const dataSet = data[index];
        const option = dataSet?.[1];
        const inlineStyle = {
            top: (style.top as number) + LISTBOX_PADDING,
        };

        if (Object.hasOwn(dataSet, 'group')) {
            return (
                <ListSubheader key={dataSet.key} component="div" style={inlineStyle}>
                    {dataSet.group}
                </ListSubheader>
            );
        }

        return (
            <div component="div" {...dataSet[0]} style={inlineStyle}>
                {renderOption(props, option)}
            </div>
        );
    };

    const useResetCache = (data: any) => {
        const ref = React.useRef<VariableSizeList>(null);
        React.useEffect(() => {
            if (ref.current != null) {
                ref.current.resetAfterIndex(0, true);
            }
        }, [data]);
        return ref;
    };

    const ListboxComponent = React.forwardRef<HTMLDivElement, React.HTMLAttributes<HTMLElement>>(
        function ListboxComponent(props, ref) {
            const { children, ...other } = props;
            const itemData: React.ReactElement[] = [];
            (children as React.ReactElement[]).forEach(
                (item: React.ReactElement & { children?: React.ReactElement[] }) => {
                    itemData.push(item);
                    itemData.push(...(item.children || []));
                }
            );

            const theme = useTheme();
            const smUp = useMediaQuery(theme.breakpoints.up('sm'), {
                noSsr: true,
            });
            const itemCount = itemData.length;
            const itemSize = smUp ? 36 : 48;

            const getChildSize = (child: React.ReactElement) => {
                if (Object.hasOwn(child, 'group')) {
                    return 48;
                }

                return itemSize;
            };

            const getHeight = () => {
                if (itemCount > 8) {
                    return 8 * itemSize;
                }
                return itemData.map(getChildSize).reduce((a, b) => a + b, 0);
            };

            const gridRef = useResetCache(itemCount);

            return (
                <>
                    <RecentClearAll
                        show={clear?.show ?? false}
                        text={clear?.text ?? 'Clear All'}
                        onClick={clear?.onClear ?? noop}
                        testId="autocomplete-clear-id"
                        recentSearchText={clear?.recentSearchText ?? 'Your Recent Searches'}
                    />
                    <div ref={ref}>
                        <OuterElementContext.Provider value={other}>
                            <VariableSizeList
                                itemData={itemData}
                                height={getHeight() + 2 * LISTBOX_PADDING}
                                width="100%"
                                ref={gridRef}
                                outerElementType={OuterElementType}
                                innerElementType="ul"
                                itemSize={(index) => getChildSize(itemData[index])}
                                overscanCount={5}
                                itemCount={itemCount}
                            >
                                {renderRow}
                            </VariableSizeList>
                        </OuterElementContext.Provider>
                    </div>
                    <SeeMoreButton
                        onSeeMore={seeMore?.onSeeMore}
                        show={seeMore?.show}
                        text={seeMore?.text}
                        options={options}
                        setScrollTopPosition={setScrollTopPosition}
                        listboxRef={listboxRef}
                    />
                </>
            );
        }
    );

    return (
        <>
            <LineLoader show={loading ?? false} />
            <Autocomplete
                open
                id={id}
                disableListWrap
                className={className}
                loading={loading}
                noOptionsText={noOptionsText}
                PopperComponent={fullWidth ? StyledPopper : Popper}
                ListboxComponent={ListboxComponent}
                options={options}
                groupBy={renderGroupBy}
                getOptionLabel={renderOptionLabel}
                renderOption={(props, option, state) => [props, option, state.index] as React.ReactNode}
                renderGroup={(params) => params as any}
                onChange={onChange}
                renderInput={renderInput}
                onInputChange={onInputChange}
                value={value}
                onFocus={onFocus ?? noop}
                inputValue={inputValue}
                filterOptions={
                    isFilterEnabled
                        ? undefined // Use default filtering behavior
                        : (options) => options
                }
                freeSolo
                onKeyDown={onKeypress}
            />
        </>
    );
};

export default NsAutocomplete;
