import React, { ReactElement, useCallback, useRef, useState } from 'react';

import { useOnClickOutside } from '../hooks/useOnClickOutside';
import { NsIcon } from '../icon';
import {
    DropdownButton,
    DropdownContainer,
    DropdownHead,
    DropdownList,
    DropdownOptionContainer,
} from './NsDropdown.styled';
import { DropdownProps } from './types.d';

const NsDropdown: React.FC<DropdownProps> = ({
    selectedLabel,
    onSelect,
    children,
    disabled = false,
    type = 'outlined',
    dropdownBorderColor = 'accentColorB',
    closeOnMouseLeave = false,
    className,
    placeholder,
}): ReactElement => {
    const [isOpen, setIsOpen] = useState(false);
    const dropdownRef = useRef<HTMLDivElement | null>(null);

    const handleToggle = useCallback(() => {
        if (disabled) return;
        setIsOpen((x) => !x);
    }, [disabled]);

    const handleClose = useCallback(() => {
        setIsOpen(false);
    }, []);

    useOnClickOutside(dropdownRef?.current, handleClose);

    const handleOnMouseLeave = useCallback(() => {
        if (closeOnMouseLeave) {
            handleClose();
        }
    }, [closeOnMouseLeave, handleClose]);

    const handleSelect = (input: string) => {
        onSelect(input);
        setIsOpen(false);
    };

    return (
        <DropdownContainer disabled={disabled} ref={dropdownRef} className={className}>
            <DropdownHead
                data-testid="dropdown-head"
                id="dropdown-head"
                disabled={disabled}
                type={type}
                onClick={handleToggle}
            >
                <DropdownButton>{selectedLabel || placeholder}</DropdownButton>
                <NsIcon
                    testId="chevron-icon"
                    id="chevron-icon"
                    name="arrowDown"
                    color="primary"
                    cursor={disabled ? 'not-allowed' : 'pointer'}
                />
            </DropdownHead>
            {isOpen ? (
                <DropdownList
                    data-testid="dropdown-list"
                    id="dropdown-list"
                    onMouseLeave={handleOnMouseLeave}
                    borderColor={dropdownBorderColor}
                >
                    {children.map((child, key) => {
                        const value = React.isValidElement(child) ? child.props.value : child;
                        return (
                            <DropdownOptionContainer
                                data-testid={`dropdown-option-container-${key}`}
                                className="dropdown-option-container"
                                key={key}
                                onClick={() => handleSelect(value)}
                            >
                                {child}
                            </DropdownOptionContainer>
                        );
                    })}
                </DropdownList>
            ) : null}
        </DropdownContainer>
    );
};

export default NsDropdown;
