import {
    getBynderAltText,
    extractBynderDescription,
    getBasicDATWidthHeight,
    parseJsonString,
    getImageType,
    getUrlBasedOnDerivative,
    getMediaUrl,
} from './bynder-util';
import { ImageType } from './types';

describe('A. bynder-util.getBasicDATWidthHeight', () => {
    it('1. should return an scale transform with given height and width', () => {
        const result = getBasicDATWidthHeight(500, 600);
        expect(result).toEqual('io=transform:scale,width:500,height:600');
    });
    it('2. should return an fill transform with given height and width', () => {
        const result = getBasicDATWidthHeight(500, 600, 'fill');
        expect(result).toEqual('io=transform:fill,width:500,height:600');
    });
    it('3. should return a url with default width and height when default value is missing', () => {
        const result = getBasicDATWidthHeight();
        expect(result).toEqual('io=transform:scale,width:900,height:800');
    });
});

describe('B. bynder-util.parseJsonString', () => {
    it('1. should successfully parse a valid JSON string', () => {
        const jsonString = '{"name": "John", "age": 30}';
        expect(parseJsonString(jsonString)).toEqual({ name: 'John', age: 30 });
    });
    it('2. should return an empty object when parsing an empty string', () => {
        const jsonString = '';
        expect(parseJsonString(jsonString)).toEqual({});
    });
});

describe('C. bynder-util.getBynderAltText', () => {
    it('1. should return the value of textMetaproperties if available', () => {
        const obj = {
            image: [{ description: 'description', textMetaproperties: [{ name: 'SEO_Tags', value: 'SEO tag value' }] }],
        };
        expect(getBynderAltText('image')(obj)).toEqual('SEO tag value');
    });

    it('2. should return an empty string if textMetaproperties is not available', () => {
        const obj = {};
        expect(getBynderAltText('image')(obj)).toEqual('');
    });
});

describe('D. bynder-util.extractBynderDescription', () => {
    it('1. should return the description if available', () => {
        const obj = { image: [{ description: 'description' }] };
        expect(extractBynderDescription('image')(obj)).toEqual('description');
    });

    it('2. should return an empty string if description is not available', () => {
        const obj = {};
        expect(extractBynderDescription('image')(obj)).toEqual('');
    });
});

describe('E. bynder-util.getImageType', () => {
    it('1. should return transformBaseUrl if url contains transformBaseUrl', () => {
        const url =
            'https://media.nuskin.com/transform/PDP_Key_Ingredients_Image/3ea706a3-770e-4b66-b112-427ef840f4b6/mail-40th-240207-1';
        const fileName = 'mail-40th-240207-1';
        const files = {
            transformBaseUrl: {
                url: 'https://media.nuskin.com/transform/PDP_Key_Ingredients_Image/3ea706a3-770e-4b66-b112-427ef840f4b6/mail-40th-240207-1',
            },
        };
        expect(getImageType(url, fileName, files)).toEqual(ImageType.transformBaseUrl);
    });

    it('2. should return customTransformation if url contains customTransformation', () => {
        const url =
            'https://media.nuskin.com/transform/3ea706a3-770e-4b66-b112-427ef840f4b6/mail-40th-240207-1?io=overlay:text,content:abc,font:Open Sans,size:64,color:FFFFFFFF,gravity:center,padding:20,angle:0';
        const fileName = 'mail-40th-240207-1';
        const files = {
            customTransformation: {
                url: 'https://media.nuskin.com/transform/3ea706a3-770e-4b66-b112-427ef840f4b6/mail-40th-240207-1',
            },
        };
        expect(getImageType(url, fileName, files)).toEqual(ImageType.customTransformation);
    });

    it('3. should return key if present in file', () => {
        const url =
            'https://media.nuskin.com/transform/PDP_Key_Ingredients_Image/3ea706a3-770e-4b66-b112-427ef840f4b6/mail-40th-240207-1';
        const fileName = 'mail-40th-240207-1';
        const files = {
            base: {
                url: 'https://media.nuskin.com/transform/PDP_Key_Ingredients_Image/3ea706a3-770e-4b66-b112-427ef840f4b6/mail-40th-240207-1',
            },
        };
        expect(getImageType(url, fileName, files)).toEqual('base');
    });
    it('4. should return defaultPreset if key not present in files', () => {
        const url =
            'https://media.nuskin.com/transform/PDP_Key_Ingredients_Image/3ea706a3-770e-4b66-b112-427ef840f4b6/mail-40th-240207-1';
        const fileName = 'mail-40th-240207-1';
        const files = {
            base: {
                url: 'https://media.nuskin.com/transform/s_Image/3ea706a3-770e-4b66-b112-427ef840f4b6/mail-40th-240207-1',
            },
        };
        expect(getImageType(url, fileName, files)).toEqual(ImageType.defaultPreset);
    });
});

describe('F. bynder-util.getUrlBasedOnDerivative', () => {
    it('1. should append suffix to custom transformed url', () => {
        const url =
            'https://media.nuskin.com/transform/PDP_Key_Ingredients_Image/3ea706a3-770e-4b66-b112-427ef840f4b6/mail-40th-240207-1?io=overlay:text,content:abc,font:Open Sans,size:64,color:FFFFFFFF,gravity:center,padding:20,angle:0';
        const suffix = 'io=transform:fill,width:90,height:90';
        const derivedUrl = getUrlBasedOnDerivative(url, ImageType.customTransformation, suffix);
        expect(derivedUrl).toEqual(`${url}&${suffix}`);
    });

    it('2. should append suffix to base url', () => {
        const url =
            'https://media.nuskin.com/transform/PDP_Key_Ingredients_Image/3ea706a3-770e-4b66-b112-427ef840f4b6/mail-40th-240207-1';
        const suffix = 'io=transform:fill,width:90,height:90';
        const derivedUrl = getUrlBasedOnDerivative(url, ImageType.transformBaseUrl, suffix);
        expect(derivedUrl).toEqual(`${url}?${suffix}`);
    });

    it('3. should return url as it is', () => {
        const url =
            'https://media.nuskin.com/transform/PDP_Key_Ingredients_Image/3ea706a3-770e-4b66-b112-427ef840f4b6/mail-40th-240207-1';
        const suffix = 'io=transform:fill,width:90,height:90';
        const derivedUrl = getUrlBasedOnDerivative(url, ImageType.defaultPreset, suffix);
        expect(derivedUrl).toEqual(url);
    });
});

describe('G. bynder-util.getMediaUrl', () => {
    it('1. should return url if asset is public', () => {
        const url =
            'https://media.nuskin.com/transform/PDP_Key_Ingredients_Image/3ea706a3-770e-4b66-b112-427ef840f4b6/mail-40th-240207-1';
        const isPublic = true;
        const previewUrls = [
            'https://media.nuskin.com/transform/PDP_Key_Ingredients_Image/3ea706a3-770e-4b66-b112-427ef840f4b6/mail-40th-240207-1',
        ];
        const mediaUrl = getMediaUrl({ url } as any, isPublic, previewUrls);
        expect(mediaUrl).toEqual(url);
    });
    it('2. should return Url if asset is not public and filetype not mentioned', () => {
        const url =
            'https://media.nuskin.com/transform/PDP_Key_Ingredients_Image/3ea706a3-770e-4b66-b112-427ef840f4b6/mail-40th-240207-1';
        const isPublic = false;
        const previewUrls = [
            'https://media.nuskin.com/transform/PDP_Key_Ingredients_Image/3ea706a3-770e-4b66-b112-427ef840f4b6/mail-40th-240207-1',
        ];
        const mediaUrl = getMediaUrl({ url } as any, isPublic, previewUrls);
        expect(mediaUrl).toEqual(url);
    });

    it('3. should return previewUrl if asset is video and is not public', () => {
        const url =
            'https://media.nuskin.com/transform/PDP_Key_Ingredients_Image/3ea706a3-770e-4b66-b112-427ef840f4b6/mail-40th-240207-1';
        const isPublic = false;
        const previewUrls = [
            'https://media.nuskin.com/transform/PDP_Key_Ingredients_Image/3ea706a3-770e-4b66-b112-427ef840f4b6/mail-40th-240207-1',
        ];
        const mediaUrl = getMediaUrl({ url, type: 'VIDEO' } as any, isPublic, previewUrls);
        expect(mediaUrl).toEqual(previewUrls[0]);
    });
    it('4. should return url if asset is video and is public', () => {
        const url =
            'https://media.nuskin.com/transform/PDP_Key_Ingredients_Image/3ea706a3-770e-4b66-b112-427ef840f4b6/mail-40th-240207-1';
        const isPublic = true;
        const previewUrls = [''];
        const mediaUrl = getMediaUrl({ url, type: 'VIDEO' } as any, isPublic, previewUrls);
        expect(mediaUrl).toEqual(url);
    });
    it('5. should return empty string if selected is empty object', () => {
        const mediaUrl = getMediaUrl({} as any, false, []);
        expect(mediaUrl).toEqual('');
    });
    it('6. should return empty string if selected is undefined', () => {
        const mediaUrl = getMediaUrl(undefined, false, []);
        expect(mediaUrl).toEqual('');
    });
});
